

;*********** [ Blei-Akku-Lader fr 12 und 24 Volt  ********************
;
;   File name   :  sla.asm
;   AUTHOR      :  Hanno Grff
;   VERSION     :  1.1
;   DATE        :  June 2007
;
;   Compiler    :  Atmel AVR Assembler
;
;	Change log  :
;	
;
;**********************************************************************


; Include-Files
.include "tn26def.inc"
.include "bc.inc"
.include "sla.inc"


;************************* Interrupt-Vectoren *************************

.cseg
	.org 0
	 rjmp POWERON     			;Reset handler
	 rjmp nix ;EXT_INT0			;IRQ0 handler
	 rjmp nix ;PIN-Change
	 rjmp nix ;TIM1_CMP1A  		;Timer1 compare match
	 rjmp nix ;TIM1_CMP1B		;Timer1 compare Match
	 rjmp nix ;TIM1_OVF     	;Timer1 overflow handler
	 rjmp TIM0_OVF     			;Timer0 overflow handler
	 rjmp nix ;USI_Start  		;USI Start handler
	 rjmp nix ;USI_OVF	   	    ;USI Overflow handler
	 rjmp nix ;EE_RDY 	   		;EEPROM Ready handler
	 rjmp nix ;ANA_COMP     	;Analog Comparator handler
	 rjmp nix ;ADC_Complete		;
nix:
	reti


txtAkku:
 	.db  "Akku 12V:",0				;10 Bytes
 	.db  "Akku 24V:",0

txtMode:
 	.db  " 1. Io",0,0				;8 Bytes! mu gerade Zahl sein!
 	.db  " 2. Uo",0,0				;
 	.db  " 3. Ut",0,0

txtAmpere:
 	.db  "2,00 A",0,0				;8 Bytes!
 	.db  "2,50 A",0,0
 	.db  "3,00 A",0,0
 	.db  "3,50 A",0,0
 	.db  "4,00 A",0,0
 	.db  "4,50 A",0,0
 	.db  "5,00 A",0,0
 	.db  "5,50 A",0,0
 	.db  "6,00 A",0,0
 	.db  "6,50 A",0,0
 	.db  "7,00 A",0,0
 	.db  "7,50 A",0,0
 	.db  "8,00 A",0,0

txtSonstiges:
	.db	 "       ",0
	.db	 "  Ende!",0


PowerOn:
	cli									; Interrupts ausschalten
	ldi		R16, RAMEND					; Stack initialisieren
	out	 	SP, R16

	
	;PINB1 (PWM), lcd_enable und LCD_RS = Ausgang
	ldi 	UL, (1<< PINB1) | (1<<PIN_LCD_ENABLE) | (1<<PIN_LCD_RS)
 	out 	DDRB, UL
	
	;PIN_LCDs (Daten) = Ausgang
	ldi 	UL, (1<<PIN_LCD_D4) | (1<<PIN_LCD_D5) | (1<<PIN_LCD_D6) | (1<<PIN_LCD_D7)
 	out 	DDRA, UL


	ldi		UL, (1 << PLLE)				; PLL aktivieren d.h. asyncroner Takt bei PWM
	out		PLLCSR, UL					; 1 Mhz * 64 = 64 Mhz


waitpll:								; Takt setzen
	in 		UL, PLLCSR
	sbrs	UL, PLOCK
	rjmp	waitpll

	in 		UL,PLLCSR
	ldi		r17, (1 << PCKE)
	or		UL, r17
	OUT		PLLCSR, UL

	;LCD initialisieren
	rcall	lcd_init
	rcall 	lcd_clear


	;"Akku:" ausgeben				
	ldi   	ZL,Low(txtAkku*2)  			; Label "Akku 12 V: "
 	ldi   	ZH,High(txtAkku*2)
 	rcall	lcd_home
	rcall 	lcd_print
	
	ldi   	ZL,Low(txtAmpere*2)  		; Label "Stromladen: "
 	ldi   	ZH,High(txtAmpere*2)
	ldi		AL,0xA						; 1.Zeile 11. Stelle
	rcall	lcd_cursor					; Cursor setzen
	rcall 	lcd_print
	
	; Spannungen in Register einlesen
	ldi		UL, low(V_Charge_fast)		; Ladeschluspannung (12V) einlesen
	mov		FAST_V_L,UL					; Wert in Registern speichern
	ldi		UL, high(V_Charge_fast)
	mov		FAST_V_H,UL

	ldi		UL, low(V_Charge_Trickle)	; Erhaltungsladespannung einlesen
	mov		TRICKLE_V_L, UL				; in Registern speichern
	ldi		UL, high(V_Charge_Trickle)	
	mov		TRICKLE_V_H, UL				

	;PWM-initialisieren
	ldi		UL, 159				    	; OCR1C setzen 159 = 25khz/50khz
	out		OCR1C, UL
	ldi		UL, 10						; unten anfangen
	out		OCR1A,UL

	ser		UL							; Flags setzen
	clr		UH

	
;**************************************************************************************************************
; Wenn Taste 1 gedrckt, dann wenn Akku an ist, starten
; Wenn Taste 1 gedrckt und kein Akku an, dann Volt hochsetzen
; Wenn Taste 2 gedrckt, Strom hochsetzen
;**************************************************************************************************************

start:	
	push	UL
	push	UH
	rcall   get_voltage					; Spannung messen	
	pop		UH
	pop		UL
	ldi		AL,100						; willkrlicher Startwert, ob eine Spannung
	clr		AH	
	cp		AV_L,AL						; gemessen werden kann <=> 3 Volt
	cpc		AV_H,AH
	brsh	akku_on
	rcall	print_voltage
	
	sbis	PINB, taste1				; ist Taste1 gedrckt
	rcall 	wait100ms					; warten
	sbis  	PINB, taste1				; Taste (noch) gedrckt
	rcall 	voltage						; Akku-Spannung einstellen
	rcall	wait100ms					; 
	rcall	wait100ms					
	sbis	PINB, taste2				; Taste2 gedrckt
	rcall 	Wait100ms					; warten
	sbis	PINB, taste2				; Taste (noch) gedrckt
	rcall	strom						; Akku-Strom einstellen
	rcall	wait100ms					; warten
	rjmp	start						; zurck
	
; In erster LCD-Zeile Akkutyp 12 od. 24 V anzeigen
voltage:
	rcall	lcd_home					; Cursor setzen
	ldi   	ZL,Low(txtAkku*2)  			; Label "Akku 12 V: " adressieren
	ldi   	ZH,High(txtAkku*2)
	tst		UL							; Ist Flag = 0
	breq	v1							; dann 12 Volt ausgeben
	adiw	ZH:ZL,10					; sonst Adressierung auf 24 V
v1:	rcall 	lcd_print					; auf LCD ausgeben
	com		UL							; Flag 'umdrehen'
	ret

; und gewhlten Ladestrom
strom:
	ldi		AL,0xA						; 1.Zeile 11. Stelle
	rcall	lcd_cursor					; Cursor setzen
	ldi   	ZL,Low(txtAmpere*2)  		; Label "Stromladen: "
 	ldi   	ZH,High(txtAmpere*2)
	
	cpi		UH,12						; Ist Max erreicht (=8,0 Amp) ?
	brlo	s0							; wenn nein, weiter bei s0
	clr		UH							; wenn ja, Flag = 0
	rjmp	s3							; Wert ausgeben
	
s0:	inc		UH							; sonst Tastendruck UH=UH+1

    mov		AL,UH
s2: adiw	ZH:ZL,8						; pro Zhlwert in UH
	dec		AL							; zur Speicherstelle
	brne	s2							; 8 Bytes addieren

s3:	rcall	lcd_print					; Ausgabe
	ret


;  Akku angeschaltet, d. h. auf Start warten
Akku_on:
	push	UL
	push	UH
	rcall	get_voltage					; nochmal Spannung messen	
	pop		UH
	pop		UL
	rcall	print_voltage				; Spannung ausgeben
	ldi		AL,100						; ist Spannung immer noch grer wie Hysterese
	clr		AH
	cp		AV_L,AL
	cpc		AV_H,AH
	brlo	start						; wenn kleiner, ganz zurck zu Anfang
										; sonst auf Tastendruck warten zum Loslegen
	sbis	PINB, Taste1				; nchsten Befehl berspringen wenn Taste1 nicht gedrckt
	rcall	Wait100ms
	sbic	PINB, Taste1				; nchsten Befehl berspringen wenn Taste noch gedrckt
	rjmp	akku_on						; => zum Laden ----->>

a0:	sbis	PINB,Taste1					; warten bis Taste frei
	rjmp	a0						


;*************************************************************************************************
;
; 1. Initialisierung 12 oder 24 Volt
;
;*************************************************************************************************
		
	;1. Strom
	ldi		AL, low(I_Charge)			
	ldi		AH, high(I_Charge)			
	ldi		BL, I_Step
	tst		UH							; ist Flag UH (Strom) = 0 = 2 Amp
	breq	i2							; dann gleich weiter mit Laden

i1:	add		AL, BL						; sonst pro Zhler I_Step addieren
	adc		AH, ZERO					; ggf. Carry addieren
	dec		UH							; Zhler dekrementieren
	brne	i1							; in AH:AL steht nun der gewhlte Ladestrom


i2:	mov 	I_Charge_L,AL				; in Registern dauerhaft
	mov 	I_CHARGE_H,AH				; fr ADC-Vergleiche speichern

;	2. Spannung	
	mov		V_FLAG, UL					; Spannung in Register sichern >0=12V 0=24V
	tst		UL							; ist Flag (Spannung) <> 0 dann gleich zum Laden
	brne	c1							; zum Laden
	
	
	lsl		FAST_V_L					; wenn 24 Volt, Spannung verdoppeln
	rol		FAST_V_H					; Ladeschluspannung
	lsl		TRICKLE_V_L					; Erhaltungsladung
	rol		TRICKLE_V_H
	lsr		I_Charge_H					; Strom halbieren
	ror 	I_CHARGE_L


;*************************************************************************************************
;
; 1. Laden mit konstantem Strom bis Spannung bei 14,8 V bzw. 29,6 V
;
;*************************************************************************************************


	; "1. Stufe:" ausgeben				
c1:
	ldi		AL,0xA						; 1.Zeile 11. Stelle
	rcall	lcd_cursor
 	ldi   	ZL,Low(txtMode*2)  			; Label "Stromladen: "
 	ldi   	ZH,High(txtMode*2)
 	rcall	lcd_print		
	
	clr		Zero						; Zhler setzen, alle 256 Durchlufe ...
	clr		BL							; Additionsregister
	clr		BH							; fr Strommesswerte
	clr		CL
	clr		CH							; CH als Zhler verwenden

	rcall   start_pwm					; PWM starten
	
c2:	rcall   get_current					; Strom messen, Rckgabe in AV_H:AV_L
	add		BL, AV_L					; aufaddieren
	adc		BH, AV_H
	adc		CL, Zero					; Null + Carry addieren

	mov		UL, I_CHARGE_L				; Vergleichswerte und
	mov		UH, I_CHARGE_H				
	rcall	adj_pwm						; AV_H:AV_L an Reglung bergeben
		
	dec		CH							; Zhler = Zhler -1
	tst		CH
	brne	c2							; wenn nicht 0, zurck zu c2

	mov		AV_L,BH						; Durchschnittswert aus 256 Durchlufen
	mov		AV_H,CL						; BL fllt weg = Division durch 256
	
	rcall	print_current				; LCD-Ausgabe Strom
	
	rcall	stop_pwm					; PWM aus 
	rcall  	get_voltage					; Spannung messen
	rcall	start_pwm					; PWM wieder ein
	
	rcall	print_voltage				; und LCD-Ausgabe Spannung
		
	cp		AV_L, FAST_V_L				; Spannung = 14,8V bzw. 29,6V
	cpc		AV_H, FAST_V_H				
 	brsh	vcf							; wenn >=, dann nchste Stufe

	clr		BL							; Additionsregister wieder auf 0
	clr		BH
	clr		CL
	clr		CH							; Zhler neu setzen

	;sbis	PINB, Taste1				; Wenn Taste gedrckt,
	;rjmp	Poweron						; Programm abbrechen
		
c3:	rjmp	c2							; und zurck


;*************************************************************************************************
;
; 2. Weiterladen fr 60 min mit konstanter Spannung bei 14,8 V
;
;*************************************************************************************************
	
vcf:									
	;2. Stufe auf LCD ausgeben				
	ldi		AL,0xA						; 1.Zeile 11. Stelle
	rcall	lcd_cursor
 	ldi   	ZL,Low(txtMode*2)  			; Label 
 	ldi   	ZH,High(txtMode*2)
	adiw	ZH:ZL,8
 	rcall	lcd_print	
	
	ldi   	AL,0x49 					; Cursor von LCD auf 2. Zeile 9. Stelle
 	rcall 	lcd_cursor
	ldi   	ZL,Low(txtSonstiges*2)  	; Leerstring zur berdeckung
 	ldi   	ZH,High(txtSonstiges*2)		; der bisherigen Stromanzeige
	rcall	lcd_print	

	clr		t_msec						; Zeit-Zhler Milli-Sekunden
	clr		t_sec						; Zeit-Zhler Sekunden
	clr		t_min						; Zeit-Zhler Minuten 

	clr		BL							; Additionsregister
	clr		BH							; auf 0 setzen
	clr		CL
	clr		CH							; Zhler auf 0 setzen									

	ldi		UL, (1<<CS02) | (1<<CS00)	; Timer/Counter0, prescaler auf 1/1024
	out		TCCR0, UL
	ldi		UL, (1<<TOIE0)				; Timer-Overflow0 setzen
	out		TIMSK,UL					; ab jetzt 1 Stunde hochzhlen
	sei									; Interrupts ermglichen

vcf1:
	rcall	get_voltage					; Spannung messen, Rckgabe AV_L, AV_H
	
	add		BL, AV_L					; aufaddieren
	adc		BH, AV_H
	adc		CL, Zero					; Null + Carry addieren

	mov 	UL, FAST_V_L				; Max-Werte holen Fast-Volt (14,8 V)
	mov		UH, FAST_V_H 				; Low-und High-byte
	rcall   adj_pwm						; an Regelung bergeben
		
	dec		CH							; Zhler = Zhler -1
	tst		CH
	brne	vcf1						; wenn nicht 0, zurck zu vcf1

	mov		AV_L,BH						; Durchschnittswert aus 256 Durchlufen
	mov		AV_H,CL						; BL fllt weg = Division durch 256
	rcall	print_voltage				; 1/Sek Spannung ausgeben
	cpi		t_min,60					; wenn 60 Minuten um
	breq	vct							; weiter mit Trickle-Charge

	rcall	print_minutes				; Minutenanzeige

	clr		BL							; Additionsregister wieder auf 0
	clr		BH
	clr		CL
	clr		CH							; Zhler neu setzen									

	;sbis	PINB, Taste1				; Wenn Taste gedrckt,
	;rcall	wait100ms
	;sbis	PINB, Taste1
	;rjmp	Poweron						; Programm abbrechen

	rjmp	vcf1


;*************************************************************************************************
;
; 3. Erhaltungsladung min mit konstanter Spannung bei 13,8 V/27,6V fr alle Ewigkeit
;
;*************************************************************************************************

vct:
	ldi		AL,0xA						; 1.Zeile 11. Stelle
	rcall	lcd_cursor	
 	ldi   	ZL,Low(txtMode*2)  			; Label
 	ldi   	ZH,High(txtMode*2)
	adiw	ZH:ZL,16
 	rcall	lcd_print		

	ldi   	AL,0x49 					; Cursor von LCD auf 2. Zeile 9. Stelle
 	rcall 	lcd_cursor
	ldi   	ZL,Low(txtSonstiges*2)  	; Leerstring zur berdeckung
 	ldi   	ZH,High(txtSonstiges*2)		; der bisherigen Stromanzeige
	adiw	ZL:ZH,8						; "Ende!" ausgeben
	rcall	lcd_print	

	clr		BL							; Additionsregister
	clr		BH							; auf 0 setzen
	clr		CL
	clr		CH

vct1:	
	rcall	get_voltage					; Spannung messen, Rckgabe AV_L, AV_H
	
	add		BL, AV_L					; aufaddieren
	adc		BH, AV_H
	adc		CL, Zero					; Null + Carry addieren

	mov 	UL, TRICKLE_V_L				; Max-Werte holen Trickle-Volt (13,8 V)
	mov		UH, TRICKLE_V_H 			; Low-und High-byte
	rcall   adj_pwm						; an Regelung bergeben
		
	dec		CH							; Zhler = Zhler -1
	tst		CH
	brne	vct1						; wenn nicht 0, zurck zu vct1

	mov		AV_L,BH						; Durchschnittswert aus 256 Durchlufen
	mov		AV_H,CL						; BL fllt weg = Division durch 256
	rcall	print_voltage				; 1/Sek Spannung ausgeben

	clr		BL							; Additionsregister wieder auf 0
	clr		BH
	clr		CL
	clr		CH							; Zhler neu setzen									

	;sbis	PINB, Taste1				; Wenn Taste gedrckt,
	;rjmp	Poweron						; Programm abbrechen

	rjmp	vct1
	

;*************************************************************************************************
;
; Timer0-Interrupt
; jede Sekunde Flag auf 1 setzen => LCD wird aktualisiert
; nach 60 Minuten Flag auf 2 setzen => weiter bei Trickle-Charge
;
;*************************************************************************************************


TIM0_OVF:								; Zhlt bis 10 Minuten
	in		sreg_storage, SREG			; Inhalt von SREG zwischenspeichern
	inc 	t_msec						; Milli-Sek. erhhen
	cpi		t_msec,4					; sind es 4 (3,8 Overflows/sec)
	brlo	t0_end						; wenn nein, dann Schleife
	clr		t_msec						; Millisekunden auf 0
	inc		t_sec						; und Sekunden erhhen
	ldi		FLAG,1						; Flag zu Anzeige setzen, d.h. jede Sek 1mal LCD aktualisieren
	cpi		t_sec,57					; sind es 60 Sekunden?
	brlo	t0_end						; wenn nicht, dann Schleife
	clr		t_sec						; Sekunden auf 0
	inc		t_min						; und Minute erhhen (59768 ms)
T0_end:
	out		SREG, sreg_storage			; Inhalt von SREG wieder herstellen
	reti


;*************************************************************************************************
;
; ADC-Handler, Eingabe: ADMUX = UL Rckgabe: AV_H:AV_l
;
;*************************************************************************************************

get_current:
	ldi		UL, ADC_Diff_Channel
	rjmp	get_measure			

get_voltage:
	ldi		UL, ADC_V_Channel

get_measure:
	cli									; Whrend Messung, Interrupts verhindern
	out		ADMUX, UL					; ADC Channel setzen
	rcall	wait1ms						; warte 1ms
	clr		AV_L
	clr		AV_H						; Additionsregister lschen
	
	ldi		UL, (1<< ADEN) 				; 
	out		ADCSR,UL
	
	sbi		ADCSR,ADSC					; 1. Start = Dummymessung
m_dummy:								
	sbis	ADCSR,ADIF
	rjmp	m_dummy

	ldi		UH,32 						; Schleifenzhler auf 32

m_start:
	sbi		ADCSR, ADSC					; 2. Start

m_wait:
	sbic	ADCSR, ADSC
	rjmp	m_wait						; Warte bis Messung erfolgt ist (ADSC wird 0)

	in		UL, ADCL					; Messungen zwischenspeichern
	add		AV_L, UL					; und
	in		UL, ADCH
	adc		AV_H, UL					; Werte aufaddieren

	dec		UH
	brne	m_start						; wiederholen bis Zhler=0
	
	ldi		UH, 5 						; Zhler fr's Schieben, 5 mal schieben = geteilt durch 32
m_avg:	
	lsr		AV_H
	ror		AV_L						; 
	dec		UH							; Zhler=Zhler-1
	brne	m_avg						;	 
	
	cbi		ADCSR,ADEN
	sei									; Interrupts wieder zulassen
	ret									; 
	

;*************************************************************************************************
;
; PWM-Handler: Eingabe Spannung in AV_H:AV_l und Vergleichswert in UH:UL
;
;*************************************************************************************************

;PWM-Regelung
adj_pwm:
	cp		AV_L,UL						; Vergleich Register, Register
	cpc		AV_H,UH						; Vergleich Register, Register + Carry
	breq	f_end						; wenn gleich, dann Ende
	cp		AV_L,UL						; nochmal vergleichen
	cpc		AV_H,UH			
	brsh	f_dec						; springe bei grer zu dec	

f_inc:									; ansonsten erhhen
	in		UL, OCR1A					; aktuellen Wert einlesen
	inc		UL							; UL=UL+1
	cpi		UL, 159						; ist UL>= OCR1C
	brsh	f_end						; dann Ende
	out		OCR1A, UL					; sonst UL nach OCR1A
	rjmp 	f_end					

f_dec:	
	in		UL, OCR1A					; aktuellen Wert einlesen
	dec		UL							; OCR1A > 0 ?
	cpi		UL,1
	brlo	f_end						; wenn Carry 0 dann vorher zu ende
	out		OCR1A, UL

f_end:
	ret


; Start and Stop PWM
start_PWM:
	sbi		PORTB,PWM_PIN
	sbi		DDRB, PWM_PIN	
	ldi 	UL, (1<< COM1A1)|(1 << PWM1A)
	out 	TCCR1A, UL						
	tst		V_Flag						; wenn V_FLAG = 0 dann 24V
	breq	p1				
	; bei 12 V 50 khz
	ldi 	UL,   (1 << CS12)			; bei 12 = 64 Mhz/ 8/159=50 khz
	out 	TCCR1B, UL					
	ret						
												
p1: ; bei 24V 25 khz	
	ldi 	UL,   (1 << CS12)|(1<<CS10)	; 12+10  = 64 Mhz/16/159=25 khz
	out 	TCCR1B, UL					
	ret						
												
												
stop_PWM:
	cbi		PORTB, PWM_PIN				
	sbi		DDRB,  PWM_PIN			
	ldi		UL, (0 << PWM1A)
	out		TCCR1A, UL					; PWM1A auf null setzen
	ret


;*******************[ Warteschleifen ]*******************************************************

wait1s: ;1002,54
	push		UL
	ldi			UL,10
n1:	rcall		wait100ms
	dec			UL
	brne		n1
	pop			UL
	ret

wait100ms:; 100,54ms
	push		UL
	ldi			UL,10
n2:	rcall		wait10ms
	dec			UL
	brne		n2
	pop			UL
	ret


Wait5ms:; 5,03ms
	push		UL
	ldi			UL,5
	rjmp		n3
Wait10ms: ;10,04ms
	push		UL
	ldi			UL,10
n3:	rcall		wait1ms
	dec			UL
	brne		n3
	pop			UL
	ret

wait1ms:
	push		UL
	ldi			UL,9
n4:	rcall		wait100us
	dec			UL
	brne		n4
	pop			UL
	ret

wait50us:	;48us 
	push		UL
	ldi			UL,14
	rjmp		n5
wait100us:;	100us
	push		UL
	ldi			UL,32
n5:	dec			UL
	brne		n5
	pop			UL
	ret


;********************************************************************************
; Spannung auf LCD ausgeben
; Werte AV_H:AV_L
; 2. Zeile 1. Stelle
;**********************************************************************************

Print_voltage:										
 
	ldi   	AL,0x40  						; Cursor von LCD auf 2. Zeile 1 Stelle
 	rcall 	lcd_cursor
 
 	mov   	AL, AV_L						; Messung speichern
 	mov   	AH, AV_H
 	add		AL, AV_L
 	adc		AH, AV_H
 	add		AL, AV_L
 	adc		AH, AV_H						; jetzt ist der dreifache Wert in AH:AL
	
	rcall 	bin2ascii						; Wert in AH:AL in Ascii umrechnen
 	
	push 	AL								; zuerst 1er Stelle sichern
 	push	AH                            	; 10er Stelle
 	push	BL								; 100er Stelle
 	mov   	AL,BH							; in BH steht 1000er Stelle
 	rcall	lcd_data						; ausgeben
 	pop 	AL								; 100er Stelle holen
 	rcall 	lcd_data                     	; ausgeben
 	ldi   	AL,','							; ein Komma zwischensetzen
 	rcall 	lcd_data                     	; ausgeben
 	pop   	AL		                        ; 10er Stelle holen
 	rcall 	lcd_data                     	; ausgeben
 	pop   	AL								; 1er Stelle holen
 	rcall 	lcd_data                     	; ausgeben
 	ldi   	AL,' '							; Leerstelle dranhngen
 	rcall 	lcd_data                     	; ausgeben
 	ldi   	AL,'V'							; V fr Volt dranhngen
 	rcall 	lcd_data                     	; ausgeben
	ret 
 

;**********************************************************************************
; Strom auf LCD ausgeben 2A = 134 Steps = ca. 134 * 15 = 2010
; Werte AV_H:AV_L
;**********************************************************************************

Print_Current:										
 
 	ldi   	AL,0x49 						;Cursor von LCD auf 2. Zeile 9. Stelle
 	rcall 	lcd_cursor
  	
 	mov  	AL, AV_L						; Messung speichern
 	mov   	AH, AV_H
	lsl		AL								; 4 mal linksschieben = mal 16
	rol		AH
	lsl		AL
	rol		AH								; 
	lsl		AL
	rol		AH								; 
	lsl		AL
	rol		AH								; 
	;sub		AL, AV_L						; und von 16 wieder 1 abziehen	
	;sbc		AH,	AV_H
	
	;in AV_H:AV_L steht jetzt der Wert in mA
	rcall 	bin2ascii						; Wert in AH:AL in Ascii umrechnen
 											; 1er Stelle in Al wird verworfen
 	push	AH                            	; 10er Stelle 
 	push	BL								; 100er Stelle
 	ldi		AL, '0'							; 1000 Stelle immer mit 0 ausgeben
	rcall	lcd_data						; ausgeben
	mov   	AL,BH							; in BH steht 1000er Stelle
 	rcall	lcd_data						; als 100 ausgeben
 	ldi   	AL,','							; ein Komma zwischensetzen
 	rcall 	lcd_data                     	; ausgeben
	pop 	AL								; 100er Stelle vom Stack holen nach AL
 	rcall 	lcd_data                     	; ausgeben
 	pop   	AL		                        ; 10er Stelle vom Stack holen
 	rcall 	lcd_data                     	; ausgeben
 	ldi   	AL,' '							; Leerstelle dranhngen
 	rcall 	lcd_data                     	; ausgeben
 	ldi   	AL,'A'							; A fr Ampere dranhngen
 	rcall 	lcd_data                     	; ausgeben
 	ret


;**********************************************************************************
; Zeit auf LCD ausgeben
; Werte AV_H:AV_L
;**********************************************************************************

Print_Minutes:
 
 	ldi   	AL,0x4A 						;Cursor von LCD auf 2. Zeile 10. Stelle
 	rcall 	lcd_cursor
  	
 	ldi  	AL, 60
 	sub   	AL, t_min
	clr		AH	

	;in AL steht jetzt die verbeibende Zeit
	rcall 	bin2ascii						; Wert in AH:AL in Ascii umrechnen
 											; 1er Stelle in Al wird verworfen
 	push	AL
 	mov		AL, AH							; 10 Stelle ausgeben
	rcall	lcd_data						; ausgeben
	pop 	AL								; 1er Stelle holen
 	rcall 	lcd_data                     	; ausgeben
 	ldi   	AL, ' '		                    ; Leerzeichen
 	rcall 	lcd_data                     	; ausgeben
 	ldi   	AL,'m'							; und min
 	rcall 	lcd_data                     	; anfgen
 	ldi   	AL,'i'							; 
 	rcall 	lcd_data                     	
 	ldi   	AL,'n'							; 
 	rcall 	lcd_data                     	
	ret


;*************************************************************************
; Binrwert in Ascii-Code wandeln 
; Eingabe: AH:AL, Rckgabe in  CL, BH:BL, AH:AL = 5 digits (ASCII)
;*************************************************************************

bin2ascii:
	ldi  	CL, -1 + '0'
bcd1: 
	inc  	CL
    subi 	AL, low(10000)
    sbci 	AH, high(10000)
    brcc 	bcd1
    ldi  	BH, 10 + '0'

bcd2:
	dec		BH
    subi 	AL, low(-1000)
    sbci 	AH, high(-1000)
    brcs 	bcd2
    ldi  	BL, -1 + '0'

bcd3:
	inc  	BL
    subi 	AL, low(100)
    sbci 	AH, high(100)
    brcc 	bcd3
    ldi  	AH, 10 + '0'

bcd4:
	dec  	AH
    subi 	AL, -10
    brcs 	bcd4
    subi 	AL, -'0'

	ret



;***************************************************************************
;*
;* LCD-Routinen  4-Bit
;*                                             	
;* PIN_LCD_Enable	= PB5									
;* PIN_LCD_RS		= PB6									
;* PIN_LCD_D4		= PA4									
;* PIN_LCD_D5		= PA5									
;* PIN_LCD_D6 		= PA6									
;* PIN_LCD_D7		= PA7									
;*																
;****************************************************************************


;sendet Datenbyte an das LCD
lcd_data:

	sbi		PortB, PIN_LCD_RS
	push	AL		          	   		;"Sicherungskopie" 2.Nibble
    andi  	AL, 0b11110000         		; unteres Nibble auf Null setzen
	out		PORTA,AL
	rcall 	lcd_enable             		; Enable-Routine aufrufen
    pop   	AL                	   		; 2. Nibble
    swap  	AL				   	   		; untere 4 bits (Nibble) nach oben	
  	andi  	AL, 0b11110000         		; untere Hlfte auf Null setzen
	out		PORTA,AL
	rcall 	lcd_enable             		; Enable-Routine aufrufen
    rcall 	wait50us              		; warte
    ret                            		; zurck zum Hauptprogramm
	

 ;sendet einen Befehl an das LCD
lcd_command:                            ; wie lcd_data, nur ohne RS zu setzen
    		
	cbi		PORTB, PIN_LCD_RS
	push	AL
    andi	AL, 0b11110000
	out		PORTA,AL
	rcall	lcd_enable
    pop		AL
	swap	AL
	andi	AL, 0b11110000
	out		PORTA,AL
	rcall	lcd_enable
    rcall	wait50us
    ret


 ;erzeugt den Enable-Puls
lcd_enable:
    
 	sbi 	PORTB, PIN_LCD_Enable    	; Enable high
    nop                          		; 3 Taktzyklen warten
	nop
	nop
	cbi		PORTB, PIN_LCD_Enable     	; Enable wieder low
	ret                          		; und wieder zurck

 
 ;Initialisierung
lcd_init:
    rcall  	wait1s

	ldi	    AL, 0b0011000         		;
    out		PORTB, AL              		;
	rcall	lcd_enable             		; 1. muss 3mal hintereinander gesendet
    rcall	wait5ms						;    werden zur Initialisierung
    rcall	lcd_enable             		; 2.
    rcall	wait5ms
    rcall	lcd_enable             		; und 3!
    rcall	wait5ms
	
	ldi		AL,	0b00100000				; 4bit Modus einstellen
	out		PORTB,AL
	rcall	lcd_enable
	rcall	Wait5ms	

	ldi	    AL, 0b00101000         		; 4bit-Modus, 2 Zeilen
    rcall	lcd_command					; Befehl senden
	ldi		AL, 0b00001100         		; LCD on
    rcall	lcd_command					; Befehl senden
	ldi		AL, 0b00000100         		; Entry mode
    rcall	lcd_command
    ret

 ;Sendet den Befehl zur Lschung des Displays
lcd_clear:
    ldi		AL, 0b00000001         		; Display lschen
    rcall	lcd_command
    rcall	Wait5ms
    ret
 ;Setzt Cursor-Position auf Anfang
lcd_home:
	ldi		AL, 0b00000010         		; Cursor an Anfang
   	rcall	lcd_command
   	rcall	wait5ms
   	ret	

; Erwartet die Position in AL			
lcd_cursor:
	sbr		AL, 0b10000000         		; Bit 7 setzen
	rcall	lcd_command
	rcall	wait5ms
    ret

lcd_print:
	lpm									; Laden
	tst		R0							; ist Zeichen =0
	breq	lcd_pend					; gehe zu end
	mov		AL,R0						; sonst Zeichen nach AL
	rcall	lcd_data					; Ausgabe
	adiw	ZL,1              			; Zeiger in ZL = ZL + 1(word)
	rjmp	lcd_print				
lcd_pend:
 	ret


